@@ -1,6 +1,6 @@
 """Detect zmq version"""
 #-----------------------------------------------------------------------------
-#  Copyright (C) 2011 Brian Granger, Min Ragan-Kelley
+#  Copyright (C) PyZMQ Developers
 #
 #  This file is part of pyzmq, copied and adapted from h5py.
 #  h5py source used under the New BSD license
@@ -56,6 +56,11 @@ def test_compilation(cfile, compiler=None, **compiler_attrs):
         else:
             # allow for missing UB arch, since it will still work:
             lpreargs = ['-undefined', 'dynamic_lookup']
+    if sys.platform == 'sunos5':
+        if platform.architecture()[0]=='32bit':
+            lpreargs = ['-m32']
+        else: 
+            lpreargs = ['-m64']
     extra = compiler_attrs.get('extra_compile_args', None)
 
     objs = cc.compile([cfile],extra_preargs=cpreargs, extra_postargs=extra)
