@@ -1,16 +1,6 @@
 # -*- coding: utf8 -*-
-#-----------------------------------------------------------------------------
-#  Copyright (c) 2010 Brian Granger, Min Ragan-Kelley
-#
-#  This file is part of pyzmq
-#
-#  Distributed under the terms of the New BSD License.  The full license is in
-#  the file COPYING.BSD, distributed as part of this software.
-#-----------------------------------------------------------------------------
-
-#-----------------------------------------------------------------------------
-# Imports
-#-----------------------------------------------------------------------------
+# Copyright (C) PyZMQ Developers
+# Distributed under the terms of the Modified BSD License.
 
 import sys
 import time
@@ -28,9 +18,6 @@ try:
 except:
     from Queue import Queue
 
-#-----------------------------------------------------------------------------
-# Tests
-#-----------------------------------------------------------------------------
 
 class TestSocket(BaseZMQTestCase):
 
@@ -54,6 +41,14 @@ class TestSocket(BaseZMQTestCase):
         s.close()
         ctx.term()
 
+    def test_bind_unicode(self):
+        s = self.socket(zmq.PUB)
+        p = s.bind_to_random_port(unicode("tcp://*"))
+
+    def test_connect_unicode(self):
+        s = self.socket(zmq.PUB)
+        s.connect(unicode("tcp://127.0.0.1:5555"))
+
     def test_bind_to_random_port(self):
         # Check that bind_to_random_port do not hide usefull exception
         ctx = self.Context()
@@ -138,8 +133,9 @@ class TestSocket(BaseZMQTestCase):
             if sopt.startswith((
                 'ROUTER', 'XPUB', 'TCP', 'FAIL',
                 'REQ_', 'CURVE_', 'PROBE_ROUTER',
+                'IPC_FILTER',
                 )):
-                # fail_unroutable is write-only
+                # some sockopts are write-only
                 continue
             try:
                 n = p.getsockopt(opt)
@@ -276,11 +272,11 @@ class TestSocket(BaseZMQTestCase):
         ctx = self.Context()
         s = ctx.socket(zmq.PUB)
         s.close()
-        self.assertRaises(zmq.ZMQError, s.bind, b'')
-        self.assertRaises(zmq.ZMQError, s.connect, b'')
-        self.assertRaises(zmq.ZMQError, s.setsockopt, zmq.SUBSCRIBE, b'')
-        self.assertRaises(zmq.ZMQError, s.send, b'asdf')
-        self.assertRaises(zmq.ZMQError, s.recv)
+        self.assertRaisesErrno(zmq.ENOTSOCK, s.bind, b'')
+        self.assertRaisesErrno(zmq.ENOTSOCK, s.connect, b'')
+        self.assertRaisesErrno(zmq.ENOTSOCK, s.setsockopt, zmq.SUBSCRIBE, b'')
+        self.assertRaisesErrno(zmq.ENOTSOCK, s.send, b'asdf')
+        self.assertRaisesErrno(zmq.ENOTSOCK, s.recv)
         del ctx
     
     def test_attr(self):
@@ -396,6 +392,38 @@ class TestSocket(BaseZMQTestCase):
                 except AttributeError:
                     pass
             s.close()
+    
+    def test_shadow(self):
+        p = self.socket(zmq.PUSH)
+        p.bind("tcp://127.0.0.1:5555")
+        p2 = zmq.Socket.shadow(p.underlying)
+        self.assertEqual(p.underlying, p2.underlying)
+        s = self.socket(zmq.PULL)
+        s2 = zmq.Socket.shadow(s.underlying)
+        self.assertNotEqual(s.underlying, p.underlying)
+        self.assertEqual(s.underlying, s2.underlying)
+        s2.connect("tcp://127.0.0.1:5555")
+        sent = b'hi'
+        p2.send(sent)
+        rcvd = self.recv(s2)
+        self.assertEqual(rcvd, sent)
+    
+    def test_shadow_pyczmq(self):
+        try:
+            from pyczmq import zctx, zsocket, zstr
+        except Exception:
+            raise SkipTest("Requires pyczmq")
+        
+        ctx = zctx.new()
+        ca = zsocket.new(ctx, zmq.PUSH)
+        cb = zsocket.new(ctx, zmq.PULL)
+        a = zmq.Socket.shadow(ca)
+        b = zmq.Socket.shadow(cb)
+        a.bind("inproc://a")
+        b.connect("inproc://a")
+        a.send(b'hi')
+        rcvd = self.recv(b)
+        self.assertEqual(rcvd, b'hi')
 
 
 if have_gevent:
@@ -431,5 +459,3 @@ if have_gevent:
             s.close()
             self.assertEqual(len(w), 1)
             self.assertEqual(w[0].category, UserWarning)
-
-
